//
// Developed by the Microsoft Security Engineering Center (MSEC)
// Copyright 2008-2009, Microsoft Corporation
//
//	Microsoft Public License (Ms-PL)
//	This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//
//	Definitions
//		The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law. A "contribution" is the original software, or any additions or changes to the software. A "contributor" is any person that distributes its contribution under this license. "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//	Grant of Rights
//		(A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//		(B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//	Conditions and Limitations
//		(A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks. 
//		(B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically. 
//		(C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software. 
//		(D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license. 
//		(E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees, or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
//

#include "stdafx.h"
#include <stdio.h>
#include "debugger.h"

DWORD GetHash(char* apiName, unsigned int n)
{
    if(n > 32) return 0; 

    DWORD hash = 0;

    while (*apiName)
        hash = ((hash<<(32-n))|(hash>>n))+(*apiName++);

    return hash;
}

//
// Return API name given a hash value
//
BOOL ScanExportTableForHash(char* dllName, DWORD dwRotationCount, DWORD dwHashValue, char* OUT outBuffer)
{
    #define RVA(offset) ((DWORD)h+(offset))

    HMODULE h = LoadLibraryA(dllName);

    if (!h)
    {
        return FALSE;
    }

    // Get the dllName as a filename
    char * dllFilename = dllName, *pos;
    while ((pos = strchr(dllFilename,'\\')) != NULL)
	dllFilename = pos+1;
   
    IMAGE_NT_HEADERS *nt;
    unsigned short ofs = * (unsigned short*) RVA(0x3c);
    nt = (IMAGE_NT_HEADERS*) RVA(ofs);

    // Check if there are some exports
    if (nt->OptionalHeader.DataDirectory[0].Size > 0)
    {
        IMAGE_EXPORT_DIRECTORY *ied = (IMAGE_EXPORT_DIRECTORY*) RVA(nt->OptionalHeader.DataDirectory[0].VirtualAddress);
        DWORD *sym = (DWORD *) RVA(ied->AddressOfNames);
    
        for (unsigned int t=0;t<ied->NumberOfNames;++t)
        {
            char* funcName = (char*) RVA(sym[t]);
            if(dwHashValue == GetHash(funcName, dwRotationCount))
            {
                sprintf_s(outBuffer, MAX_PATH, "%s (%s)", funcName, dllFilename);
                FreeLibrary(h);
                return TRUE;
            }
        }
    }

    FreeLibrary(h);
    return FALSE;
}

HRESULT CALLBACK
ror(PDEBUG_CLIENT5 pClient, PCSTR pszArgs)
{
    HRESULT hr = E_FAIL;

	DEBUGGER_CONTROLS objDebugger( pClient );

	if ( !objDebugger.fInitialized )
		return E_INVALIDARG;

    //
    // Parse the args
    //
    char  seps[]    = " \t";
    char* token     = NULL;
    char *nextToken = NULL;

    int   argc = 0;
    char* argv[4]; // we take max 4 args.
    char* args = (char*)pszArgs;

    // Establish string and get the first token:
    token = strtok_s( args, seps, &nextToken);

    // While there are tokens in args
    while ((token != NULL))
    {
        // Get next token
        if (token != NULL)
        {
            argv[argc++] = token;
            if(argc > 4) break;
            token = strtok_s( NULL, seps, &nextToken);
        }
    }

    if (argc < 1)
    {
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "Usage: %s [-n <Rotation Count>] [-c <Value>]\n", "!ror");                
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "Example: %s 0x0E8AFE98\n", "!ror"); 
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "         Get API name for hash value 0x0E8AFE98 using default rotation count 13.\n"); 
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "Example: %s -n 15 0xA9F72DC9\n", "!ror");        
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "         Get API name for hash value 0xA9F72DC9 using rotation count 15.\n"); 
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "Example: %s -n 15 -c WinExec\n", "!ror");        
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "         Get hash value for WinExec using rotation count 15.\n"); 
        
        return 0;
    }

    int     i = 0;
    DWORD   dwRotationCount = 13; // Default 
    DWORD   dwHashValue = 0;
    char*   pszApiName  = NULL;

    if (strcmp(argv[i],"-n") == 0)
    {
        i++;
        dwRotationCount = atoi(argv[i]);
        i++;
    }

    if (strcmp(argv[i],"-c") == 0)
    {
        pszApiName = argv[++i];
    }
    else
    {    
        //
        // Handle various formats such as 0x1234, 0n1234, 1234, eip, @eip, etc.
        //
        // Get the windbg-style extension APIS.   
        ExtensionApis.nSize = sizeof (ExtensionApis);
        objDebugger.pDebugControl->GetWindbgExtensionApis64(&ExtensionApis);

        ULONG64 dwHashValue64;    
        PCSTR   pcszRemainder = NULL;
        if (!GetExpressionEx(argv[i], &dwHashValue64, &pcszRemainder)) return E_INVALIDARG;
        dwHashValue = (ULONG)dwHashValue64;
    }

    if(pszApiName)
    {
        objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "0x%08X\n", GetHash(pszApiName, dwRotationCount));        
    }
    else
    {
        char* commonDLLs[] = 
        {
            "kernel32.dll",
            "urlmon.dll",
            "wininet.dll",
            "winhttp.dll",
            "advapi32.dll",
            "ntdll.dll",
            "gdi32.dll",
            "user32.dll",
            "shell32.dll",
            "shlwapi.dll",
            NULL
        };

        char apiNameOut[MAX_PATH];
        int idx = 0;
        while ( *(commonDLLs + idx) != NULL ) 
        {
            if(ScanExportTableForHash(commonDLLs[idx++], dwRotationCount, dwHashValue, apiNameOut) == TRUE) 
            {
                objDebugger.pDebugControl->Output(DEBUG_OUTPUT_NORMAL, "%s\n", apiNameOut);
                break;
            }
        }   
    }

    return hr;

}